import React, { useState, useEffect } from 'react';
import ReactDOM from 'react-dom';
import { json, select, extent, scaleLinear, scaleBand } from 'd3';

const jsonUrl = 'https://raw.githubusercontent.com/freeCodeCamp/ProjectReferenceData/master/global-temperature.json';
const width = 1200;
const height = 450;
const margin = { top: 40, bottom: 95, left: 110, right: 60 };
const mlist = [ "January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December" ];

const App = () => {
  const [data, setData] = useState(null);
  
  useEffect(() => {
  	json(jsonUrl).then(setData);
  }, []);
  
  if (!data) {
    return <div>Loading..</div>;
  }
  
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  
  const baseTemp = data.baseTemperature;
  const variation = data.monthlyVariance
  const xScale = scaleLinear().domain(extent(variation, d => d.year)).range([0, innerWidth]);
  const yScale = scaleBand().domain(variation.map(v => v.month)).range([0, innerHeight]);
 	const opScale = scaleLinear().domain([0, 2*baseTemp]).range([0, 1]);
  
  return (
  	<>
      <div id='title'>
        <h1>Monthly Global Land-Surface Temperature</h1>
        <h2>{variation[0].year} - {variation[variation.length-1].year}: base temperature {baseTemp}</h2>
      </div>
      <svg width={width} height={height}>
      	<g transform={`translate(${margin.left},${margin.top})`}>
          {
          	xScale.ticks().map(tickValue => <g className='tick' transform={`translate(${xScale(tickValue)},${innerHeight + 5})`}>
                                 <line 
                                   y1={-5}
                                   stroke='black'
                                   />
                                 <text
                                   style={{textAnchor: 'middle'}}
                                   dy="0.71em"
                                   >{tickValue}</text>
                               </g>)
        	}
          <g id='x-axis' transform={`translate(0,${innerHeight})`}>
          <line 
            x2={innerWidth}
            stroke='black'
            />
            <text
              id='axis-label'
              x={innerWidth/2}
              style={{textAnchor: 'middle'}}
              y={40}
              >Years</text>
          </g>
          {
          	yScale.domain().map(tickValue => <g className='tick' transform={`translate(0,${yScale(tickValue)})`}>
                                 <text
																		dy="0.32em"
                                   style={{textAnchor: 'end'}}
                                   x={-5}
                                   >{mlist[tickValue-1]}</text>
                                 <line 
                                   x1={-5}
                                   stroke='black'
                                   />
                               </g>) 
          }
          <g id='y-axis'>
          <line 
            y2={innerHeight}
            stroke='black'
            />
            <text
              id='axis-label'
              transform={`translate(-80,${innerHeight / 2}) rotate(-90)`}
              style={{textAnchor: 'middle'}}
              >Months</text>
          </g>
          {
          	variation.map(d => <g 
                                 className='bars' 
                                 transform={`translate(${xScale(d.year)},${yScale(d.month)})`}
                                 onMouseEnter={() => {
                              		select('#tooltip').append('span').text(d.year + ' - ' + mlist[d.month-1]);
                              		select('#tooltip').append('span').text(baseTemp + d.variance + '°C');
                              		select('#tooltip').append('span').text(d.variance + '°C');
                              		select('#tooltip').style('left', xScale(d.year) + margin.left + 'px');
																  select('#tooltip').style('top', yScale(d.month)  + margin.top + 'px'); 
                              		select('#tooltip').style('visibility','visible');
                            		}}
                                 onMouseLeave={() => {
                            			select('#tooltip').selectAll('span').remove();
                              		select('#tooltip').style('visibility','hidden');
                            		}}
                                 >
                          <rect
                            width={innerWidth / (variation[variation.length-1].year - variation[0].year)}
                            height={yScale.bandwidth()}
                            fill='red'
                            fill-opacity={opScale(baseTemp + d.variance)}
                            />
                          </g>)
          }
        </g>
      </svg>
      <div id='tooltip'></div>
      </>
  );
}

ReactDOM.render(<App />, document.getElementById('root'))